#include <math.h>
#include <string.h>
#include "pcsolver.h"
#include "globalc.h"
#include "global.h"
#include "globalt.h"
#include "globmat.h"
#include "globmatt.h"
#include "globmatc.h"
#include "elemswitcht.h"
#include "gmatrix.h"
#include "gtopology.h"
#include "loadcase.h"
#include "mechprint.h"
#include "transprint.h"
#include "mtsolver.h"
#include "elemswitch.h"
#include "dloadcase.h"

/**
   function solves partially coupled thermo-hydro-mechanical time-dependent problem   
   transport problems are linear or nonlinear
   mechanical analysis is always nonlinear
*/
void solve_pcouplprob ()
{
  
  switch (Cp->tnlinsol){
  case newtonc:{
    newton_raphson_parcoupl (0);
    break;
  }
  default:{
    print_err("unknown solver of nonlinear equation system is required",__FILE__,__LINE__,__func__);
  }
  }
}


/**
   function solves partially coupled thermo-hydro-mechanical time-dependent problem
   by the Newton-Raphson method
   
   @param lcid - load case id
   
   11.6.2008
*/
void newton_raphson_parcoupl (long lcid)
{
  switch (Tp->tprob){
  case nonstationary_problem:{
    //  linear nonstationary transport problem
    //  nonlinear mechanical problem
    newton_raphson_parcoupl_lin (lcid);
    break;
  }
  case nonlinear_nonstationary_problem:{
    //  nonlinear nonstationary transport problem
    //  nonlinear mechanical problem
    newton_raphson_parcoupl_nonlin (lcid);
    break;
  }
  default:{
    print_err("unknown TRFEL problem type is required",__FILE__,__LINE__,__func__);
  }
  }
}

/**
   function solves system of linear TRFEL algebraic and non-linear MEFEL equations by Newton-Raphson method
   for time-dependent problems

   @param lcid - load case id
   
   15.7.2005 JK, corrected by TKr 5.4.2007
*/
void newton_raphson_parcoupl_lin (long lcid)
{
  int mefel_convergence;
  long i,ii,j,k,nm,nt,ini,nsts,jj;
  double zero,dt,dtmin,dtdef,end_time,alpha,s;
  double *dr,*fb,*r,*d,*f,*fp,*fi,*fl,*p,*lhst,*tdlhst,*rhst,*lhsb,*lhstb,*tdlhstb;
  double norf, norfa, err,norf_old,norf_new;  
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  FILE *aux;
  
  /*
  Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  Mp->path = path;
  Mp->filename=name;
  Mp->suffix=suffix;
  sprintf (fname,"%s%s.bac",path, name);

  aux = fopen (fname,"w");
  */

  //  maximum number of iterations in inner loop
  ini = Cp->niilnr;
  //  required norm of vector of unbalanced forces
  err = Mp->errnr;
  //  number of mechanical degrees of freedom
  nm=Ndofm;
  //  number of transport degrees of freedom
  nt=Ndoft;

  /* *************************** */
  /*  vectors of transport part  */
  /* *************************** */

  //  vector of nodal values
  lhst=Lsrst->give_lhs (lcid);
  //  vector of time derivatives of nodal values
  tdlhst=Lsrst->give_tdlhs (lcid);
  //  vector of the right hand side
  rhst=Lsrst->give_rhs (lcid);


  //  array containing predictors
  d = new double [nt];
  nullv (d,nt);
  //  auxiliary vector
  p = new double [nt];
  nullv (p,nt);
  //  backup of the lhs vector
  lhstb = new double [nt];
  nullv (lhstb,nt);
tdlhstb = new double [nt];	  
nullv (tdlhstb,nt);

  /* **************************** */
  /*  vectors of mechanical part  */
  /* **************************** */

  //  vector of nodal displacements
  r = Lsrs->give_lhs (lcid);
  //  vector of nodal prescribed forces
  f = Lsrs->give_rhs (lcid);

  //  vector of increments of nodal displacements
  dr = new double [nm];
  nullv (dr,nm);
  //  vector of prescribed forces from the previous step
  fp = new double [nm];
  nullv (fp,nm);
  //  vector of prescribed force loads, it does not contain forces caused by temperature, etc.
  fl = new double [nm];
  nullv (fl,nm);
  //  vector of internal forces
  fi = new double [nm];
  nullv (fi,nm);
  //  auxiliary force vector
  fb = new double [nm];
  nullv (fb,nm);
  //  backup of the nodal displacements
  lhsb = new double [nm];
  nullv (lhsb,nm);
  
  
  
  //  coefficient of trapezoidal method
  alpha=Tp->alpha;
  zero=Tp->zero;
  
  //  maximum number of iterations in inner loop
  ini = Mp->niilnr;
  //  required norm of vector of unbalanced forces
  err = Mp->errnr;

  //driving time from METR
  //  starting time
  Cp->time=Cp->timecon.starttime ();
  //  time increment
  dt=Cp->timecon.initialtimeincr ();
  //  end time
  end_time = Cp->timecon.endtime ();
  
  //  mechanical time controller is rewritten by coupled time controller
  Mp->time = Cp->time;
  Mp->timecon.take_values (Cp->timecon);
  //  transport time controller is rewritten by coupled time controller
  Tp->time = Cp->time;
  Tp->timecont.take_values (Cp->timecon);
  
  //  initial temperature is approximated to the mechanical integration points
  approximation_inittemper ();
  //  initiation of mechanical material models
  Mm->initmaterialmodels();

  //  nodes - integration points interpolation in TRFEL part
  approximation ();
  
  actual_previous_nodval ();
  
  //  initiation of transport material models
  Tm->initmaterialmodels();

  compute_req_valt (0);

  // **************************
  //  main iteration loop  ***
  // **************************  
  i=0;
  ii=0;
  //  number of successful time steps (number of steps without inner iterations)
  nsts=0;

  print_init(-1, "wt");
  print_step(lcid,ii,Mp->time,f);
  print_flush();
  print_initt(-1, "wt");
  print_stept(0,i,Tp->time,NULL);
  print_flusht();
  mefel_convergence = 1;//yes

  // *********************************************************
  //                     transport part
  // *********************************************************
  do{
    if(mefel_convergence == 0){
      Cp->timecon.oldtime ();
      Tp->timecont.oldtime ();
      if (Mesprc==1)  fprintf (stdout,"\n\n --------------------------------------------------------------------------------------------");
      if (Mesprc==1)  fprintf (stdout,"\n REDUCED TIME STEP, METR TIME STEP = %ld : METR TIME = %e, metr time increment = %e",i,Cp->time,dt);
      if (Mesprc==1)  fprintf (stdout,"\n --------------------------------------------------------------------------------------------");
      i--;
    }

    //  time update
    //new time and new time increment
    Cp->time = Cp->timecon.newtime (dt);
    Tp->time = Tp->timecont.newtime (dt);
    Tp->time = Cp->time;
    Tp->timecont.take_values (Cp->timecon);    
    i++;

    if(mefel_convergence == 1){
      if (Mesprc==1)  fprintf (stdout,"\n\n -------------------------------------------------------------------------------------------");
      if (Mesprc==1)  fprintf (stdout,"\n NEW METR TIME STEP = %ld: METR TIME = %e, metr time increment = %e",i,Cp->time,dt);
      if (Mesprc==1)  fprintf (stdout,"\n -------------------------------------------------------------------------------------------");
    }
    if (Mesprc==1)  fprintf (stdout,"\n ------------------------------------------------------------------------------------");
    if (Mesprc==1)  fprintf (stdout,"\n TRFEL TIME STEP = %ld, TRFEL TIME = %e, trfel time increment = %e",i,Tp->time,dt);
    if (Mesprc==1)  fprintf (stdout,"\n ------------------------------------------------------------------------------------");

    Tm->updateipval ();



    //  conductivity matrix
    conductivity_matrix (lcid);
    
    //  capacity matrix
    capacity_matrix (lcid);
    
    //  predictor
    for (j=0;j<nt;j++){
      p[j] = lhst[j] + (1.0-alpha)*dt*tdlhst[j];
    }
    
    //  right hand side of transport part
    trfel_right_hand_side (0,rhst,nt);
    
    //  auxiliary vector
    Kmat->gmxv (p,d);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    for (j=0;j<nt;j++){
      rhst[j] = rhst[j] - d[j];
      d[j]=tdlhst[j];
    }
    
    //  solution of the system of algebraic equations
    Tp->ssle->solve_system (Gtt,Kmat,tdlhst,rhst,Outt);
    
    
    //  backup of attained nodal values
    for (j=0;j<nt;j++){
      lhstb[j]=lhst[j];
      tdlhstb[j]=tdlhst[j];
    }
    
    //  new nodal values
    for (j=0;j<nt;j++){
      s=(1.0-alpha)*d[j]+alpha*tdlhst[j];
      lhst[j]+=dt*s;
    }

    //  physically corrected solution
    solution_correction ();    
    //  approximation of nodal values into ontegration points
    approximation ();

    actual_previous_nodval ();
    
    if (Cp->fcsolv == fullnewtonc){
      //  cleaning matrices for more free memmory
      //  cleaning of the conductivity matrix
      if (Kmat != NULL){
	delete Kmat;
	Kmat=NULL;
      }
      //  cleaning of the capacity matrix
      if (Cmat != NULL){
	delete Cmat;
	Cmat=NULL;
      }
    }

    // *********************************************************
    //  end of transport part and beginning of mechanical part
    // *********************************************************
    mefel_convergence = 1;
    //new time increment
    Mp->time = Mp->timecon.newtime (dt);

    if (Mp->time <= Tp->time){
      ii++;

      if (Mesprc==1)  fprintf (stdout,"\n\n -----------------------------------------------------------------------------------");
      if (Mesprc==1)  fprintf (stdout,"\n MEFEL TIME STEP = %ld, MEFEL TIME = %e, mefel time increment = %e",ii,Mp->time,dt);
      if (Mesprc==1)  fprintf (stdout,"\n -------------------------------------------------------------------------------");

      //  indicator of strain computation
      //  it means, strains at integration points have not been computed
      Mp->strainstate=0;
      //  indicator of stress computation
      //  it means, stresses at integration points have not been computed
      Mp->stressstate=0;
      //  indicator of computation of other array
      //  it means, stresses at integration points have not been computed
      Mp->otherstate=0;

      //  approximation of temperatures from TRFEL nodes to MEFEL integration points
      approximation_temper ();
      //  approximation of humidities from TRFEL nodes to MEFEL integration points
      approximation_humid ();
      

      //  backup of attained nodal values
      for (j=0;j<nm;j++){
	lhsb[j]=r[j];
      }

      //  computation of unbalanced forces which will be on the right hand side
      incr_internal_forces (lcid,fi);
      
      //  assembling of the right hand side (prescribed load)
      mefel_right_hand_side (lcid,f,fl);
      
      //  computation of sums of force components in particular directions
      //  it is used in special creep and consolidation models
      Mb->comp_sum (f);
      fprintf(Out, "\n# i=%ld, time=%le[s], time=%le[d], fx=%le, fy=%le, fz=%le", 
              i, Mp->time, Mp->time/86400, Mb->sumcomp[0], Mb->sumcomp[1], Mb->sumcomp[2]);
            
      for (j=0;j<nm;j++){
	fb[j]=f[j]-fp[j]+fi[j];
      }    
      
      //  assembling of tangent stiffness matrix
      stiffness_matrix (lcid);

      //  solution of K(r).v=F
      Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);
            
      //  new displacement vector
      for (j=0;j<nm;j++){
	r[j]+=dr[j];
      }
      
      //  computation of internal forces
      internal_forces (lcid,fi);
      
      norfa=ss(f,f,nm);
      for (j=0;j<nm;j++){
	fb[j] = fl[j] - fi[j];
      }

      //  norm of vector of unbalanced forces
      norf=ss(fb,fb,nm);
      if (norfa > 0.0)
	norf /= norfa;
      
      if (Mespr==1) fprintf (stdout,"\n\n Norf before inner itertion loop, norf = %e\n\n\n",norf);
	    

      j = 0;
      if (norf<err){
	nsts++;
      }      
      // iteration of unbalanced forces caused by time independent models
      //  internal iteration loop
      else{
	//-----------------------------------------------
	//convergence control added by TKr 30/07/2008
	jj = 0;
	norf_old = norf;
	//-----------------------------------------------
	for (j=0;j<ini;j++)
	  {
	    	    
	    if(Cp->fcsolv == fullnewtonc){
	      //cleaning of stiffness matrix
	      if (Smat != NULL){
		delete Smat;
		Smat=NULL;
	      }
	      //assembling of tangent stiffness matrix for fullnewton
	      stiffness_matrix (lcid);
	    }
	    
	    //  solution of K(r).v=F
	    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);
	    
	    for (k=0;k<nm;k++)
	      r[k]+=dr[k];

	    //  computation of internal forces
	    internal_forces (lcid,fi);
	    
	    //  vector of unbalanced forces
	    for (k=0;k<nm;k++)
	      fb[k]= fl[k] - fi[k];
	    
	    //  norm of vector of unbalanced forces
	    norf=ss(fb,fb,nm);
	    if (norfa > 0.0)
	      norf /= norfa;
	    
	    //if (Mespr==1) 
	    if (j%10==0)
	      fprintf (stdout,"\n Inner loop j =  %ld     norf = %e\n",j,norf);
	    
	    //-----------------------------------------------
	    //convergence control added by TKr 30/07/2008
	    norf_new = norf; 
	    if(norf_old < norf_new){
	      fprintf (stdout,"\n Divergence of inner iteation loop j = %ld is probably detected, norf = %e\n",j,norf);
	      jj++;
	    }
	    else 
	      jj = 0;
	    if (jj > 100){ 
	      fprintf (stdout,"\n\n Divergence of inner iteation loop is detected. Iteration is stopped.\n\n");
	      
	      break;
	    }
	    norf_old = norf_new;
	    //-----------------------------------------------
	    
	    if (norf<err)  break;
	  }
      }

      if (j==ini || norf>err)
	{
	  if (Mespr==1) fprintf (stdout,"\n\n Iteration process does not converge req. err = %e, reached err = %e", err,norf);

	  // equilibrium state was not reached
	  mefel_convergence = 0;

	  //  backup is used for actual MEFEL values
	  for (j=0;j<nm;j++){
	    r[j]=lhsb[j];
	  }
	  
	  //  reduction of the time increment because
	  //  inner loop was not able to enforce equilibrium
	  dt/=2.0;
	  Mp->timecon.oldtime ();
	  ii--;
	  fprintf (stdout,"\n time increment is reduced because the inner loop was not able to enforce equilibrium");
	  if (dt<dtmin){
	    fprintf (stderr,"\n\n time increment is less than minimum time increment");
	    fprintf (stderr,"\n computation fails (file %s, line %d)\n",__FILE__,__LINE__);
	    compute_req_val (lcid);
	    print_step_forced(lcid, i, Mp->time, f);
	    print_flush();
	    break;
	  }	    
	}
      else
	{
	  // equilibrium state was reached
	  fprintf (stdout,"\n\n Last inner iteration step number j = %ld,     norf = %e \n\n",j,norf);
	  mefel_convergence = 1;
	  
          //  actual total load vector becomes previous total load vector
          for (j=0;j<nm;j++){
            fp[j]=f[j];
          }

	  dtdef = Mp->timecon.actualforwtimeincr ();
	  
	  if (nsts==2){
	    dt*=2.0;
	    nsts=0;
	    fprintf (stdout,"\n\n time increment is enlarged because no inner loop was neccessary in previous 3 steps");
	  }
	  if (dt>dtdef)
	    dt=dtdef;
	  
	  //if (Mp->timecon.isitimptime ()==1)
	  //mtsolver_save (aux,r,fp,ii,Mp->time,nm,0,0);
	  
          Mb->comp_sum (fi);
          fprintf(Out, "\n# i=%ld, time=%le[s], time=%le[d], fix=%le, fiy=%le, fiz=%le", 
                  i, Mp->time, Mp->time/86400, Mb->sumcomp[0], Mb->sumcomp[1], Mb->sumcomp[2]);
	  //  update of values stored at integration points
	  Mm->updateipval();
	  compute_req_val (lcid);
	  
	  print_step(lcid, ii, Mp->time, fb);
	  print_flush();
	  
	  //backup of previous load
	  for (j=0;j<nm;j++){
	    fp[j]=f[j];
	  }  
	}
    }

    if(mefel_convergence == 0){
      //  backup is used for actual TRFEL values
      for (j=0;j<nt;j++){
	lhst[j]=lhstb[j];
	tdlhst[j]=tdlhstb[j];
      }
    }
    else{
      print_stept(0,i,Tp->time,NULL);
      print_flusht();
    }

  }while(Cp->time<=end_time);
  
  print_close ();
  print_closet ();
  
  delete [] fi;
  delete [] fb;
  delete [] fp;
  delete [] dr;
  delete [] fl;
  delete [] p;
  delete [] d;
  delete [] lhstb;
  delete [] tdlhstb;
}




/**
   Function solves system of non-linear TRFEL algebraic and non-linear MEFEL equations by Newton-Raphson method
   for time-dependent problems.

   @param lcid - load case id
   
   TKr 5.4.2007
*/
void newton_raphson_parcoupl_nonlin (long lcid)
{
  long i,ii,j,k,nm,nt,ini;
  double zero,dt,end_time,alpha;
  double *dr,*fb,*r,*d,*f,*fp,*fi,*p,*lhst,*tdlhst,*rhst;
  double *fbt,*fit,*lhst_last;
  //double s;
  double norf_last;
  double norf, norfa, err;  
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  FILE *aux;

  Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  Mp->path = path;
  Mp->filename=name;
  Mp->suffix=suffix;
  sprintf (fname,"%s%s.bac",path, name);

  aux = fopen (fname,"w");

  //  maximum number of iterations in inner loop
  ini = Tp->nii;
  //  required norm of vector of unbalanced forces
  err = Tp->err;
  //  number of mechanical degrees of freedom
  nm=Ndofm;
  //  number of transport degrees of freedom
  nt=Ndoft;

  /* *************************** */
  /*  vectors of transport part  */
  /* *************************** */
  //  left hand side
  lhst=Lsrst->give_lhs (0);
  tdlhst=Lsrst->give_tdlhs (0);
  //  right hand side
  rhst=Lsrst->give_rhs (0);

  //  array containing predictors
  d = new double [nt];
  //  auxiliary vector
  p = new double [nt];
  fbt = new double [nt];
  fit = new double [nt];
  lhst_last = new double [nt];

  //  initial values
  nullv (lhst,nt);
  nullv (tdlhst,nt);
  nullv (d,nt);
  nullv (p,nt);
  nullv (fbt,nt);
  nullv (fit,nt);
  
  /* **************************** */
  /*  vectors of mechanical part  */
  /* **************************** */
  fp = new double [nm];
  fb = new double [nm];
  fi = new double [nm];
  dr = new double [nm];

  //  initial values
  nullv (fp,nm);
  nullv (fb,nm);
  nullv (fi,nm);
  nullv (dr,nm);
 
  //  initialization phase
  r = Lsrs->give_lhs (lcid);
  f = Lsrs->give_rhs (lcid);

  alpha=Tp->alpha;
  zero=Tp->zero;
  
  //driving time from METR
  //  starting time
  Cp->time=Cp->timecon.starttime ();
  //  time increment
  dt=Cp->timecon.initialtimeincr ();
  //  end time
  end_time = Cp->timecon.endtime ();
  
  Mp->time = Cp->time;
  Mp->timecon.take_values (Cp->timecon);
  Tp->time = Cp->time;
  Tp->timecont.take_values (Cp->timecon);
  
  //  nodes - integration points interpolation
  approximation ();
  approximation_temper ();
  approximation_humid ();
  
  //  initiation of mechanical material models
  Mm->initmaterialmodels();
  //  initiation of transport material models
  Tm->initmaterialmodels();

  // **************************
  //  main iteration loop  ***
  // **************************  
  i=0;
  ii=0;
  print_init(-1, "wt");
  print_step(lcid,ii,Mp->time,f);
  print_flush();
  print_initt(-1, "wt");
  print_stept(0,i,Tp->time,NULL);
  print_flusht();

  // *********************************************************
  //                     transport part
  // *********************************************************
  do{
    
    if (Mesprc==1)  fprintf (stdout,"\n\n -----------------------------------------------------------------------------");
    if (Mesprc==1)  fprintf (stdout,"\n metr time %e,    trfel time %e,   mefel time %e",Cp->time,Tp->time,Mp->time);
    if (Mesprc==1)  fprintf (stdout,"\n ---------------------------------------------------------------------------\n");

    if (Mesprc==1)  fprintf (stdout,"\n\n provadi se TRFEL,  trfel time %e",Tp->time);
            
    Tm->updateipval ();
    i++;

    //  conductivity matrix
    conductivity_matrix (lcid);
    
    //  capacity matrix
    capacity_matrix (lcid);
    
    for (j=0;j<nt;j++){
      p[j] = lhst[j] + (1.0-alpha)*dt*tdlhst[j];
    }
    
    //  right hand side of transport part
    trfel_right_hand_side (0,rhst,nt);
    
    //  auxiliary vector
    Kmat->gmxv (p,d);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    for (j=0;j<nt;j++){
      rhst[j] = rhst[j] - d[j];
    }
    
    //  solution of the system of algebraic equations
    Tp->ssle->solve_system (Gtt,Kmat,tdlhst,rhst,Outt);

    for (j=0;j<nt;j++){
      //verze1:
      //s=(1.0-alpha)*d[j]+alpha*tdlhst[j];
      //lhst[j]+=dt*s;
      //verze2:
      lhst[j] = p[j] + alpha*dt*tdlhst[j];
    }
    
    //  physically corrected solution
    solution_correction ();    
    //  approximation of nodal values into ontegration points
    approximation ();
    
    norf_last = 1.0e20;
    //inner iteration loop//////////////////
    ////////////////////////////////////////
    for (j=0;j<ini;j++){
      
      //  physically corrected solution
      solution_correction ();
      //  approximation of nodal values into ontegration points
      approximation ();
      
      // full newton-raphson
      if (Tp->trsolv == fullnewtont){
	// matrices are computing in each inner iteration
	//  cleaning of the conductivity matrix
	if (Kmat != NULL){
	  delete Kmat;
	  Kmat=NULL;
	}
	//  cleaning of the capacity matrix
	if (Cmat != NULL){
	  delete Cmat;
	  Cmat=NULL;
	}
	//  capacity matrix
	capacity_matrix (0);
	
	//  conductivity matrix
	conductivity_matrix (0);
	
	//  auxiliary vector  K (d+(1-alpha)*dt*v)
	Kmat->gmxv (p,d);
	
	//  matrix of the system of equations
	//  C + alpha.dt.K
	Kmat->scalgm (dt*alpha);
	Kmat->addgm (1.0,*Cmat);
      }
      
      if (Tp->trestype==lrhst){
	//  right hand side of transport part
	trfel_right_hand_side (0,rhst,nt);
	// Solver computes residuum from system of equations
	Kmat->gmxv (tdlhst,fit);
	// Res. = F - K (d+(1-alpha)*dt*v) - (C + alpha*dt*K)*v 
	for (k=0;k<nt;k++){
	  fbt[k] = rhst[k] - d[k] - fit[k];
	}
      }
      
      if (Tp->trestype==fluxest){
	// Solver computes unbalanced fluxes
	internal_fluxes (fit,nt);
	//  vector of unbalanced fluxes
	for (k=0;k<nt;k++){
	  fbt[k]=fit[k];
	}
      }
      
      norf = ss (fbt,fbt,nt);
      
      if (Mesprt==1)  fprintf (stdout,"\n inner iteration %ld   error %e",j,norf);
      
      if (norf<err) break;
      
      Tp->ssle->solve_system (Gtt,Kmat,d,fbt,Outt);
      
      for (k=0;k<nt;k++){
	tdlhst[k]+=d[k];
	lhst[k]+=alpha*dt*d[k];
      }
      
      //  physically corrected solution
      solution_correction ();
      //  approximation of nodal values into ontegration points
      approximation ();
      
      //  convergence control for newton-raphson
      //  condition of decreesing of error
      if (Tp->convergcontrolt==yes){
	if (norf > norf_last){
	  for (k=0;k<nt;k++){
	    lhst[k] = lhst_last[k]; //storing results from previous inner step
	  }
	  
	  //  physically corrected solution
	  solution_correction ();
	  //  approximation of nodal values into ontegration points
	  approximation ();	  

	  if (Mesprt==1)  fprintf (stdout,"\n\n convergence control: inner iteration skiped %ld error %e\n\n",j,norf);	  
	  
	  break;
	}
	for (k=0;k<nt;k++){
	  lhst_last[k] = lhst[k]; //storing results from previous inner step
	}
	
	norf_last = norf; //storing norf from previous inner step
      }
    }

    ////////////////////////////////////////
    if (Cp->fcsolv == fullnewtonc){
      //cleaning matrices for more free memmory
      //  cleaning of the conductivity matrix
      if (Kmat != NULL){
	delete Kmat;
	Kmat=NULL;
      }
      //  cleaning of the capacity matrix
      if (Cmat != NULL){
	delete Cmat;
	Cmat=NULL;
      }
    }

    // *********************************************************
    //  end of transport part and beginning of mechanical part
    // *********************************************************
    err = Mp->errnr;
    if (Mp->time <= Tp->time){

      if (Mesprc==1)  fprintf (stdout,"\n\n provadi se MEFEL,  mefel time %e",Mp->time);
      
      approximation_temper ();
      approximation_humid ();
      
      Mm->updateipval();

      //fprintf (stdout,"\nright hand side\n\n");
      //  assembling of the right hand side (prescribed loading)
      mefel_right_hand_side (lcid,f);
            
      //  computation of sums of force components in particular directions
      //  it is used in special creep and consolidation models
      Mb->comp_sum (f);
      
      Mp->phase=1;
      //  computation of unbalanced forces which will be on the right hand side
      internal_forces (lcid,fi);
      
      for (j=0;j<nm;j++){
	fb[j]=f[j]-fp[j]+fi[j];
	fp[j]=f[j];
      }    
      
      //  assembling of tangent stiffness matrix
      //fprintf (stdout,"\nphase1 = stiffness_matrix\n\n");
      stiffness_matrix (lcid);
      
      //  solution of K(r).v=F
      Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

      //  new displacement vector
      for (j=0;j<nm;j++){
	r[j]+=dr[j];
      }
      
      Mp->phase=2;

      //  update of computed values
      internal_forces (lcid,fi);

      norfa=ss(f,f,nm);
      for (j=0;j<nm;j++){
	fb[j] = f[j] - fi[j];
      }
      //  norm of vector of unbalanced forces
      norf=ss(fb,fb,nm);
      if (norfa > 0.0)
	norf /= norfa;
      
      if (norf<err){

	// computation of values required for output    
	compute_req_val (lcid);
	//  time increment
	Mp->time = Mp->timecon.newtime ();
	Mp->timecon.take_values (Mp->timecon);
	
	if (Mp->timecon.isitimptime ()==1){
	  mtsolver_save (r,fp,ii,Mp->time,dt,nm);
	}
	
	print_step(lcid, ii, Mp->time, f);
	print_flush();
	ii++;
      }
      
      // iteration of unbalanced forces caused by time independent models
      //  internal iteration loop
      else{
	for (j=0;j<ini;j++)
	  {

	    if(Mp->nrsolv == fullnewton){
	      //cleaning of stiffness matrix
	      if (Smat != NULL){
		delete Smat;
		Smat=NULL;
	      }
	      //assembling of tangent stiffness matrix for fullnewton
	      stiffness_matrix (lcid);
	    }
	    
	    //  solution of K(r).v=F
	    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

	    for (k=0;k<nm;k++)
	      r[k]+=dr[k];
	    
	    //  computation of internal forces
	    internal_forces (lcid,fi);
	    
	    //  vector of unbalanced forces
	    for (k=0;k<nm;k++)
	      fb[k]=f[k]-fi[k];
	    
	    //  norm of vector of unbalanced forces
	    norf=ss(fb,fb,nm);
	    if (norfa > 0.0)
	      norf /= norfa;
	    
	    if (Mespr==1) fprintf (stdout,"\n Time increment %lf   inner loop j %ld     norf=%e",Mp->time,j,norf);
	    if (norf<err)  break;
	  }
	if (j==ini || norf>err)
	  {
	    if (Mespr==1) fprintf (stdout,"\n Iteration process does not converge req. err = %e, reached err = %e", err,norf);

	    // equilibrium state was not reached
	    compute_req_val (lcid);
	    //  time increment
	    Mp->time = Mp->timecon.newtime ();
	    Mp->timecon.take_values (Mp->timecon);
	    
	    if (Mp->timecon.isitimptime ()==1)
	      mtsolver_save (r,fp,ii,Mp->time,dt,nm);
	    print_step(lcid, ii, Mp->time, f);
	    print_flush();
	    ii++;
	  }
	else
	  {
	    // equilibrium state was reached
	    compute_req_val (lcid);
	    //  time increment
	    Mp->time = Mp->timecon.newtime ();
	    Mp->timecon.take_values (Mp->timecon);
	    
	    if (Mp->timecon.isitimptime ()==1)
	      mtsolver_save (r,fp,ii,Mp->time,dt,nm);
	    print_step(lcid, ii, Mp->time, f);
	    print_flush();
	    ii++;
	  }
      }
    }

    //  new time and new time increment
    Cp->time = Cp->timecon.newtime ();
    Tp->time = Cp->time;
    Tp->timecont.take_values (Cp->timecon);    
    dt = Cp->timecon.actualbacktimeincr ();
    
    print_stept(0,i,Tp->time,NULL);
    print_flusht();

  }while(Cp->time<=end_time);
  
  print_close ();
  print_closet ();
  
  delete [] fi;
  delete [] fb;
  delete [] fp;
  delete [] dr;

  delete [] p;
  delete [] d;

  delete [] fit;
  delete [] fbt;
  delete [] lhst_last;
}










/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */



/**
   function solves system of nonlinear algebraic
   equations by Newton-Raphson method
   
   @param lcid - load case id
   
   19.5.2004 JK, corrected by TKr 5.4.2007
*/
void newton_raphson_parcoupl_nonlin_old (long lcid)
{
  long i,ii,j,k,nm,nt,ini;
  double zero,dt,end_time,alpha,norfb,err;
  double *r,*dr,*d,*f,*fb,*fp,*fim,*p,*lhs,*tdlhs,*rhs;
  
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  FILE *aux;

  Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  Mp->path = path;
  Mp->filename=name;
  Mp->suffix=suffix;
  sprintf (fname,"%s%s.bac",path, name);

  aux = fopen (fname,"w");

  //  number of mechanical degrees of freedom
  nm=Ndofm;
  //  number of transport degrees of freedom
  nt=Ndoft;
  
  
  r = new double [nm];
  dr = new double [nm];

  /* *************************** */
  /*  vectors of transport part  */
  /* *************************** */
  //  array containing predictors
  d = new double [nt];
  nullv (d,nt);
  //  auxiliary vector
  p = new double [nt];
  nullv (p,nt);
  //  right hand side
  fb = new double [nt];
  nullv (fb,nt);

  /* **************************** */
  /*  vectors of mechanical part  */
  /* **************************** */
  //  vector of nodal displacements
  r = new double [nm];
  nullv (r,nm);
  //  vector of prescribed forces
  f = new double [nm];
  nullv (f,nm);
  //  vector of prescribed forces from the previous step
  fp = new double [nm];
  nullv (fp,nm);
  //  vector of internal forces
  fim = new double [nm];
  nullv (fim,nm);
  
  
  alpha=Tp->alpha;
  zero=Tp->zero;
  err=Tp->err;
  ini=Tp->nii;
  
  
  //  starting time
  Cp->time=Cp->timecon.starttime ();
  //  time increment
  dt=Cp->timecon.initialtimeincr ();
  //  end time
  end_time = Cp->timecon.endtime ();
  
  //  nodes - integration points interpolation
  approximation ();
  
  // **************************
  //  main iteration loop  ***
  // **************************
    
  Mp->time = Cp->time;
  Mp->timecon.take_values (Mp->timecon);
  Tp->time = Cp->time;
  Tp->timecont.take_values (Cp->timecon);

  i=0;
  ii=0;
  print_init(-1, "wt");
  print_step(lcid,ii,Mp->time,f);
  print_flush();
  print_initt(-1, "wt");
  print_stept(0,i,Tp->time,NULL);
  print_flusht();
  

  approximation_temper ();
  approximation_humid ();
  
  //  initiation of mechanical material models
  Mm->initmaterialmodels();
  
  // *********************************************************
  //                     transport part
  // *********************************************************
  do{
    
    if (Mesprc==1)  fprintf (stdout,"\n\n -----------------------------------------------------------------------------");
    if (Mesprc==1)  fprintf (stdout,"\n metr time %e,    trfel time %e,   mefel time %e",Cp->time,Tp->time,Mp->time);
    if (Mesprc==1)  fprintf (stdout,"\n ---------------------------------------------------------------------------\n");

    if (Mesprc==1)  fprintf (stdout,"\n\n provadi se TRFEL,  trfel time %e",Tp->time);

    lhs=Lsrst->lhs;
    tdlhs=Lsrst->tdlhs;
    rhs=Lsrst->rhs;
    
    //  conductivity matrix
    conductivity_matrix (lcid);
    
    //  capacity matrix
    capacity_matrix (lcid);
    
    //  predictor   d+(1-alpha)*dt*v
    for (j=0;j<nt;j++){
      d[j] = lhs[j] + (1.0-alpha)*dt*tdlhs[j];
    }
    
    //  right hand side of transport part
    trfel_right_hand_side (0,rhs,nt);
    
    //  auxiliary vector  K (d+(1-alpha)*dt*v)
    Kmat->gmxv (d,p);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    //  F - K (d+(1-alpha)*dt*v)
    for (j=0;j<nt;j++){
      fb[j] = rhs[j] - p[j];
    }
    
    //  solution of the system of algebraic equations
    //Kmat->solve_system (Gtt,tdlhs,fb);
    Cp->ssle->solve_system (Gtt,Kmat,tdlhs,fb,Outt);

    //  d_{i+1}=\tilda{d_{i+1}} + alpha.dt.v_{i+1}
    for (j=0;j<nt;j++){
      lhs[j] = d[j] + alpha*dt*tdlhs[j];
    }
    

    solution_correction ();
    //  approximation of nodal values into ontegration points
    approximation ();
    
    //  computation of actual fluxes
    //  fb contains unbalanced fluxes
    internal_fluxes (fb,nt);
    
    //  norm of unbalanced fluxes
    norfb = ss (fb,fb,nt);
    
    if (Mesprc==1)  fprintf (stdout,"\n%e %e",norfb,err);    
    
    if (norfb>err){
      
      //  inner iteration loop
      for (j=0;j<ini;j++){
	
	//  correction, the right hand side is created by unbalanced fluxes
	//Kmat->solve_system (Gtt,p,fb);
	Cp->ssle->solve_system (Gtt,Kmat,p,fb,Outt);

	for (k=0;k<nt;k++){
	  tdlhs[k]+=p[k];
	  lhs[k]+=alpha*dt*p[k];
	}

	solution_correction ();
	//  approximation of nodal values into ontegration points
	approximation ();
	
	//  computation of actual fluxes
	//  fb contains unbalanced fluxes
	internal_fluxes (fb,nt);
	
	//  norm of unbalanced fluxes
	norfb = ss (fb,fb,nt);
	
	if (Mesprc==1)  fprintf (stdout,"\n iterace %ld   chyba %e",j,norfb);
	
	if (norfb<err){
	  break;
	}
      }
    }
    
    print_stept(0,i,Tp->time,NULL);
    print_flusht();
    
    
    // *********************************************************
    //  end of transport part and beginning of mechanical part
    // *********************************************************

    if (Mp->time <= Tp->time){

      if (Mesprc==1)  fprintf (stdout,"\n provadi se MEFEL,  mefel time %e",Mp->time);
    
    approximation_temper ();
    approximation_humid ();
    
    
    Mm->updateipval();
    
    //  mechanical part of the problem
    lhs=Lsrs->lhs;
    rhs=Lsrs->rhs;
    
    //  assembling of the right hand side (prescribed loading)
    mefel_right_hand_side (lcid,f);
    
    //  computation of sums of force components in particular directions
    //  it is used in special creep and consolidation models
    Mb->comp_sum (f);
    
    Mp->phase=1;
    //Mm->est=eigstrain;
    //  computation of unbalanced forces which will be on the right hand side
    internal_forces (lcid,fim);
    
    
    for (j=0;j<nm;j++){
      rhs[j]=f[j]-fp[j]+fim[j];
      fp[j]=f[j];
    }    
    
    //  assembling of tangent stiffness matrix
    stiffness_matrix (lcid);
    
    //  solution of K(r).v=F
    //Smat->solve_system (Gtm,r,rhs);
    Cp->ssle->solve_system (Gtm,Smat,r,rhs,Out);

    //  new displacement vector
    for (j=0;j<nm;j++){
      lhs[j]+=r[j];
    }
    
    Mp->phase=2;
    //  update of computed values
    internal_forces (lcid,fim);

    compute_req_val (lcid);

    Mp->time = Mp->timecon.newtime ();
    Mp->timecon.take_values (Mp->timecon);

    if (Mp->timecon.isitimptime ()==1){
      mtsolver_save (lhs,fp,ii,Mp->time,dt,nm);
    }
    
    print_step(lcid, ii, Mp->time, f);
    print_flush();
    ii++;
    }

    //  new time and new time increment
    Cp->time = Cp->timecon.newtime ();
    Tp->time = Cp->time;
    Tp->timecont.take_values (Cp->timecon);
    
    dt = Cp->timecon.actualbacktimeincr ();
    i++;
    
  }while(Cp->time<=end_time);
  
  print_close ();
  print_closet ();
  
  delete [] fim;
  delete [] fb;
  delete [] fp;
  delete [] f;
  delete [] p;
  delete [] d;
  delete [] dr;
  delete [] r;

}




/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */
/* -------------------------------------------------------------- */


/**
   function solves system of nonlinear algebraic
   equations by Newton-Raphson method
   
   @param lcid - load case id
   
   19.5.2004 - modified by TKr 31.10.2005
*/
void newton_raphson_parcoupl_nonlin_new (long lcid)
{
  long i,j,k,nm,nt,ini;
  double zero,dt,end_time,alpha,norfb,err;
  double *r,*dr,*d,*f,*fb,*fp,*fim,*fi,*p,*lhs,*tdlhs,*rhs;
  
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  FILE *aux;

  Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  Mp->path = path;
  Mp->filename=name;
  Mp->suffix=suffix;
  sprintf (fname,"%s%s.bac",path, name);

  aux = fopen (fname,"w");

  //  number of mechanical degrees of freedom
  nm=Ndofm;
  //  number of transport degrees of freedom
  nt=Ndoft;
  
  
  r = new double [nm];
  dr = new double [nm];

  /* *************************** */
  /*  vectors of transport part  */
  /* *************************** */
  //  array containing predictors
  d = new double [nt];
  nullv (d,nt);
  //  auxiliary vector
  p = new double [nt];
  nullv (p,nt);
  //  right hand side
  fb = new double [nt];
  nullv (fb,nt);
  //  vector of internal fluxes
  fi = new double [nt];
  nullv (fi,nt);
  
  /* **************************** */
  /*  vectors of mechanical part  */
  /* **************************** */
  //  vector of nodal displacements
  r = new double [nm];
  nullv (r,nm);
  //  vector of prescribed forces
  f = new double [nm];
  nullv (f,nm);
  //  vector of prescribed forces from the previous step
  fp = new double [nm];
  nullv (fp,nm);
  //  vector if internal forces
  fim = new double [nm];
  nullv (fim,nm);
  
  
  alpha=Tp->alpha;
  zero=Tp->zero;
  err=Tp->err;
  ini=Tp->nii;
  
  
  //  starting time
  Cp->time=Cp->timecon.starttime ();
  //  time increment
  dt=Cp->timecon.initialtimeincr ();
  //  end time
  end_time = Cp->timecon.endtime ();
  
  //  nodes - integration points interpolation
  approximation ();
  
  // **************************
  //  main iteration loop  ***
  // **************************
    
  Mp->time = Cp->time;
  Mp->timecon.take_values (Mp->timecon);
  Tp->time = Cp->time;
  Tp->timecont.take_values (Cp->timecon);

  i=0;
  print_init(-1, "wt");
  print_step(lcid,i,Mp->time,f);
  print_flush();
  print_initt(-1, "wt");
  print_stept(0,i,Tp->time,NULL);
  print_flusht();
  

  approximation_temper ();
  approximation_humid ();
  
  //  initiation of mechanical material models
  Mm->initmaterialmodels();
  
  // *********************************************************
  //                     transport part
  // *********************************************************
  do{
    
    if (Mesprc==1)  fprintf (stdout,"\n\n -----------------------------------------------------------------------------");
    if (Mesprc==1)  fprintf (stdout,"\n metr time %e,    trfel time %e,   mefel time %e",Cp->time,Tp->time,Mp->time);
    if (Mesprc==1)  fprintf (stdout,"\n ---------------------------------------------------------------------------\n");

    if (Mesprc==1)  fprintf (stdout,"\n\n provadi se TRFEL,  trfel time %e",Tp->time);

    lhs=Lsrst->lhs;
    tdlhs=Lsrst->tdlhs;
    rhs=Lsrst->rhs;
    
    //  conductivity matrix
    conductivity_matrix (lcid);
    
    //  capacity matrix
    capacity_matrix (lcid);
    
    //  predictor   d+(1-alpha)*dt*v
    for (j=0;j<nt;j++){
      d[j] = lhs[j] + (1.0-alpha)*dt*tdlhs[j];
    }
    
    //  right hand side of transport part
    trfel_right_hand_side (0,rhs,nt);
    
    //  auxiliary vector  K (d+(1-alpha)*dt*v)
    Kmat->gmxv (d,p);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    //  F - K (d+(1-alpha)*dt*v)
    for (j=0;j<nt;j++){
      fb[j] = rhs[j] - p[j];
    }
    
    //  solution of the system of algebraic equations
    //Kmat->solve_system (Gtt,tdlhs,fb);
    Cp->ssle->solve_system (Gtt,Kmat,tdlhs,fb,Outt);

    //  d_{i+1}=\tilda{d_{i+1}} + alpha.dt.v_{i+1}
    for (j=0;j<nt;j++){
      lhs[j] = d[j] + alpha*dt*tdlhs[j];
    }

        
    //inner iteration loop
    for (j=0;j<ini;j++){
      
      solution_correction ();
      approximation ();
      
      if (Tp->trsolv == fullnewtont){
	//  capacity matrix
	capacity_matrix (0);
	
	//  conductivity matrix
	conductivity_matrix (0);
	
	//  auxiliary vector  K (d+(1-alpha)*dt*v)
	Kmat->gmxv (d,p);
	
	//  matrix of the system of equations
	//  C + alpha.dt.K
	Kmat->scalgm (dt*alpha);
	Kmat->addgm (1.0,*Cmat);
	
      }
      
      
      if (Tp->trestype==lrhst){
	Kmat->gmxv (tdlhs,fi);
	//  Res. = F - K (d+(1-alpha)*dt*v) - (C + alpha*dt*K)*v 
	for (k=0;k<nt;k++){
	  fb[k] = rhs[k] - p[k] - fi[k];
	}
      }
      if (Tp->trestype==fluxest){
	internal_fluxes (fi,nt);
	//  vector of unbalanced fluxes
	for (k=0;k<nt;k++){
	  fb[k]=fi[k];
	}
      }
      
      norfb = ss (fb,fb,nt);
      
      if (Mesprt==1)  fprintf (stdout,"\n iteration %ld   error %e",j,norfb);

      if (norfb<err){
	break;
      }

      //Kmat->solve_system (Gtt,p,fb);
      Cp->ssle->solve_system (Gtt,Kmat,p,fb,Outt);

      for (k=0;k<nt;k++){
	tdlhs[k]+=p[k];
	lhs[k]+=alpha*dt*p[k];
      }
    }
    
    print_stept(0,i,Tp->time,NULL);
    print_flusht();
    
    
    // *********************************************************
    //  end of transport part and beginning of mechanical part
    // *********************************************************

    if (Mp->time <= Tp->time){

      if (Mesprc==1)  fprintf (stdout,"\n provadi se MEFEL,  mefel time %e",Mp->time);
    
    approximation_temper ();
    approximation_humid ();
    
    
    Mm->updateipval();
    
    //  mechanical part of the problem
    lhs=Lsrs->lhs;
    rhs=Lsrs->rhs;
    
    //  assembling of the right hand side (prescribed loading)
    mefel_right_hand_side (lcid,f);
    
    //  computation of sums of force components in particular directions
    //  it is used in special creep and consolidation models
    Mb->comp_sum (f);
    
    Mp->phase=1;
    //Mm->est=eigstrain;
    //  computation of unbalanced forces which will be on the right hand side
    internal_forces (lcid,fim);
    
    
    for (j=0;j<nm;j++){
      rhs[j]=f[j]-fp[j]+fim[j];
      fp[j]=f[j];
    }    
    
    //  assembling of tangent stiffness matrix
    stiffness_matrix (lcid);
    
    //  solution of K(r).v=F
    //Smat->solve_system (Gtm,r,rhs);
    Cp->ssle->solve_system (Gtm,Smat,r,rhs,Out);

    //  new displacement vector
    for (j=0;j<nm;j++){
      lhs[j]+=r[j];
    }
    
    Mp->phase=2;
    //  update of computed values
    internal_forces (lcid,fim);

    compute_req_val (lcid);

    Mp->time = Mp->timecon.newtime ();
    Mp->timecon.take_values (Mp->timecon);

    if (Mp->timecon.isitimptime ()==1){
      mtsolver_save (lhs,fp,i,Mp->time,dt,nm);
    }
    
    print_step(lcid, i, Mp->time, f);
    print_flush();
        
    }

    //  new time and new time increment
    Cp->time = Cp->timecon.newtime ();
    Tp->time = Cp->time;
    Tp->timecont.take_values (Cp->timecon);
    
    dt = Cp->timecon.actualbacktimeincr ();
    i++;
    
  }while(Cp->time<=end_time);
  
  print_close ();
  print_closet ();
  
  delete [] fim;
  delete [] fi;
  delete [] fb;
  delete [] fp;
  delete [] f;
  delete [] p;
  delete [] d;
  delete [] dr;
  delete [] r;

}
